"""
Script Description: This script automates the download of various IMDb pages using Selenium and ChromeDriver, 
                    including handling cookies and popups, and saving the resulting HTML to local files.
					It automatically finds your localization by using service, the http://ipinfo.io API to get the country code and the dictionary to map to language acording to obtaing country code. If you don't want this, comment out this part of the script and uncomment the one at the end of this script.

Author: [afrocuban (thanks to Ivek23)]
Version: 3.1
Date: 2025-01-04
"""

import sys
import os
import logging
import threading
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException, WebDriverException
import time
import requests

# Get the current script directory
current_dir = os.path.dirname(os.path.abspath(__file__))
tmp_dir = os.path.join(current_dir, "Tmp")

# Ensure the /Tmp directory exists
if not os.path.exists(tmp_dir):
    os.makedirs(tmp_dir)

# Configure logging
log_path = os.path.join(current_dir, "python_script.log")
logging.basicConfig(
    filename=log_path,
    filemode='w',  # This ensures the log file is overwritten at the start of each run
    level=logging.DEBUG,  # Set logging to DEBUG level to capture more details
    format='%(asctime)s - %(levelname)s - %(message)s'
)

logging.debug("Starting the Python script.")

# Check if the IMDb URL is passed as a parameter
if len(sys.argv) < 2:
    logging.error("IMDb URL was not provided as a parameter.")
    sys.exit("IMDb URL was not provided as a parameter.")

imdb_url = sys.argv[1]
base_url = imdb_url.rstrip('/')

# Generate URLs and their corresponding save paths dynamically, excluding -Main page
URLS_AND_PATHS = {
    f"{base_url}/awards/": os.path.join(tmp_dir, "downpage-UTF8_NO_BOM-Awards.htm"),
    f"{base_url}/bio/": os.path.join(tmp_dir, "downpage-UTF8_NO_BOM-Bio.htm"),
    f"https://www.imdb.com/search/title/?explore=genres&role={base_url.split('/')[-1]}": os.path.join(tmp_dir, "downpage-UTF8_NO_BOM-Genres.htm"),
    f"{base_url}/?showAllCredits=true": os.path.join(tmp_dir, "downpage-UTF8_NO_BOM-Credit.htm")
}

logging.debug(f"Generated URLs and Paths: {URLS_AND_PATHS}")

# Path to chromedriver.exe
chrome_path = os.path.join(current_dir, "chromedriver.exe")

# Ensure the ChromeDriver exists
if not os.path.exists(chrome_path):
    logging.error(f"ChromeDriver not found at path: {chrome_path}")
    sys.exit(f"ChromeDriver not found at path: {chrome_path}")

# Dictionary of country codes and their respective language codes
countryLangs = {
    "CN":"zh-CN", "IN":"hi", "US":"en", "ID":"id", "BR":"pt-BR", "PK":"ur",
    "BD":"bn", "NG":"en", "RU":"ru", "JP":"ja", "MX":"es", "PH":"tl", "VN":"vi", "ET":"am", 
    "DE":"de", "EG":"ar", "TR":"tr", "IR":"fa", "CD":"fr", "TH":"th", "FR":"fr", "GB":"en", 
    "IT":"it", "MM":"my", "ZA":"zu", "KR":"ko", "ES":"es", "UA":"uk", "CO":"es", "TZ":"sw", 
    "AR":"es", "KE":"en", "PL":"pl", "SD":"ar", "DZ":"ar", "CA":"en", "UG":"en", "MA":"ar", 
    "PE":"es", "IQ":"ar", "AF":"fa", "NP":"ne", "MY":"ms", "UZ":"uz", "VE":"es", "SA":"ar", 
    "GH":"en", "YE":"ar", "KP":"ko", "TW":"zh", "SY":"ar", "MZ":"pt", "RO":"ro", "AU":"en", 
    "LK":"si", "MG":"fr", "CI":"fr", "CM":"en", "CL":"es", "NL":"nl", "BF":"fr", "NE":"fr", 
    "MW":"ny", "KZ":"kk", "EC":"es", "KH":"km", "ML":"fr", "GT":"es", "ZM":"en", "AO":"pt", 
    "SN":"fr", "ZW":"en", "CU":"es", "RW":"rw", "GR":"el", "CS":"cu", "PT":"pt", "TN":"ar", 
    "TD":"fr", "CZ":"cs", "BE":"nl", "GN":"fr", "SO":"so", "HU":"hu", "BO":"es", "BI":"fr", 
    "DO":"es", "BY":"be", "HT":"ht", "SE":"sv", "BJ":"fr", "AZ":"az", "SS":"en", "AT":"de", 
    "HN":"es", "CH":"de", "TJ":"tg", "IL":"he", "RS":"sr", "BG":"bg", "HK":"zh", "TG":"fr", 
    "LY":"ar", "JO":"ar", "PY":"es", "LA":"lo", "PG":"en", "SV":"es", "NI":"es", "ER":"aa", 
    "KG":"ky", "DK":"da", "SK":"sk", "SL":"en", "FI":"fi", "NO":"no", "AE":"ar", "TM":"tk", 
    "CF":"fr", "SG":"cmn", "GE":"ka", "IE":"en", "BA":"bs", "CR":"es", "HR":"hr", "MD":"ro", 
    "NZ":"en", "LB":"ar", "PR":"en", "PS":"ar", "LR":"en", "LT":"lt", "UY":"es", "PA":"es",
    "MR":"ar", "MN":"mn", "CG":"fr", "AL":"sq", "AM":"hy", "OM":"ar", "JM":"en", "KW":"ar", 
    "LV":"lv", "NA":"en", "MK":"mk", "BW":"en", "SI":"sl", "LS":"en", "XK":"sq", "GM":"en", 
    "GW":"pt", "GA":"fr", "SZ":"en", "MU":"en", "EE":"et", "TT":"en", "TL":"tet", "CY":"el", 
    "GQ":"es", "FJ":"en", "QA":"ar", "RE":"fr", "KM":"ar", "GY":"en", "DJ":"fr", "BH":"ar", 
    "BT":"dz", "ME":"sr", "SB":"en", "CV":"pt", "LU":"lb", "SR":"nl", "MO":"zh", "GP":"fr", 
    "MQ":"fr", "MT":"mt", "MV":"dv", "BN":"ms", "BZ":"en", "IS":"is", "BS":"en", "BB":"en", 
    "EH":"ar", "PF":"fr", "VU":"bi", "NC":"fr", "GF":"fr", "WS":"sm", "ST":"pt", "LC":"en", 
    "GU":"en", "YT":"fr", "CW":"nl", "AN":"nl", "TO":"to", "VI":"en", "GD":"en", "FM":"en", 
    "VC":"en", "KI":"en", "JE":"en", "SC":"en", "AG":"en", "AD":"ca", "IM":"en", "DM":"en", 
    "AW":"nl", "MH":"mh", "BM":"en", "GG":"en", "AS":"en", "GL":"kl", "MP":"fil", "KN":"en", 
    "FO":"fo", "KY":"en", "SX":"nl", "MF":"fr", "LI":"de", "MC":"fr", "SM":"it", "GI":"en", 
    "AX":"sv", "VG":"en", "CK":"en", "TC":"en", "PW":"pau", "BQ":"nl", "WF":"wls", "AI":"en", 
    "TV":"tvl", "NR":"na", "MS":"en", "BL":"fr", "SH":"en", "PM":"fr", "IO":"en", "FK":"en", 
    "SJ":"no", "NU":"niu", "NF":"en", "CX":"en", "TK":"tkl", "VA":"la", "CC":"ms", "TF":"fr", 
    "PN":"en", "GS":"en", "UM":"en"
}

# Function to get the country code using ipinfo.io
def get_country_code():
    try:
        response = requests.get('http://ipinfo.io/country')
        if response.status_code == 200:
            return response.text.strip()
        else:
            logging.error(f"Failed to get country code. Status code: {response.status_code}")
            return None
    except Exception as e:
        logging.error(f"Exception occurred while getting country code: {e}")
        return None

# Get the country code and corresponding language code
country_code = get_country_code()
if not country_code:
    logging.error("Failed to retrieve country code. Defaulting to English.")
    language_code = 'en'
else:
    language_code = countryLangs.get(country_code, 'en')
logging.debug(f"Country code: {country_code}, Language code: {language_code}")

# Set Chrome options
chrome_options = webdriver.ChromeOptions()
chrome_options.add_argument("--no-sandbox")
chrome_options.add_argument("--disable-dev-shm-usage")
chrome_options.add_argument("--disable-gpu")  # Disable GPU for headless mode stability
chrome_options.add_argument("--headless")  # Running Chrome in headless mode
chrome_options.add_argument(f"--lang={language_code}")
chrome_options.add_argument("--user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36")

# Function to download a page
def download_page(imdb_url, output_path, retries=3):
    logging.debug(f"Starting download for URL: {imdb_url}")
    logging.debug(f"Output path: {output_path}")

    attempt = 0
    while attempt < retries:
        attempt += 1
        logging.debug(f"Attempt {attempt} for URL: {imdb_url}")

        try:
            # Start chromedriver.exe
            service = Service(chrome_path)
            driver = webdriver.Chrome(service=service, options=chrome_options)
            logging.info(f"ChromeDriver started from: {chrome_path}")

            try:
                # Open the base URL to set the cookie
                driver.get("https://www.imdb.com")
                cookie = {'name': 'example_cookie', 'value': 'example_value', 'domain': 'imdb.com'}
                driver.add_cookie(cookie)
                logging.debug("Cookie added.")

                # Navigate to the IMDb page
                driver.get(imdb_url)
                logging.info(f"Page {imdb_url} loaded successfully.")

                # Handle "Select Your Preferences" popup with a timeout
                try:
                    popup = WebDriverWait(driver, 3).until(
                        EC.presence_of_element_located((By.XPATH, "//div[contains(@class, 'sc-kDvujY')]"))
                    )
                    logging.info("'Select Your Preferences' popup detected.")
                    accept_button = WebDriverWait(driver, 3).until(
                        EC.element_to_be_clickable((By.XPATH, "//button[@data-testid='accept-button']"))
                    )
                    accept_button.click()
                    logging.info("Clicked on the 'Accept' button to close the popup.")
                except TimeoutException:
                    logging.info("No 'Select Your Preferences' popup appeared or an error occurred. Moving on.")

                # Wait for the page to load
                WebDriverWait(driver, 30).until(
                    EC.presence_of_element_located((By.TAG_NAME, "body"))
                )

                # Click all "More" buttons
                def click_see_more_buttons():
                    while True:
                        buttons = driver.find_elements(By.XPATH, "//span[@class='ipc-btn__text']//span[contains(@class, 'ipc-see-more__text')]")
                        if not buttons:
                            logging.debug("No more 'See More' buttons found.")
                            break
                        for button in buttons:
                            try:
                                driver.execute_script("arguments[0].scrollIntoView(true);", button)  # Scroll to the button to ensure its visible
                                time.sleep(1)
                                button.click()
                                logging.info("Clicked a 'See More' button.")
                                time.sleep(1)  # Short delay to ensure the click is registered
                            except Exception as e:
                                logging.warning(f"Error clicking a 'See More' button: {e}")

                # Click the "See More" buttons
                click_see_more_buttons()

                # Retrieve the full source HTML of the page after all "More" buttons are clicked
                html_source = driver.page_source
                logging.debug(f"HTML source length: {len(html_source)}")

                # Save the HTML to a file
                with open(output_path, 'w', encoding='utf-8') as file:
                    file.write(html_source)
                logging.info(f"HTML saved to file: {output_path}")

                # If successful, break out of the retry loop
                break

            except WebDriverException as e:
                logging.error(f"An error occurred while processing {imdb_url}: {e}")
                driver.quit()

        except Exception as e:
            logging.error(f"An error occurred: {e}")

        finally:
            driver.quit()
            logging.info("Browser closed.")

# Create and start threads for each URL
threads = []
for url, save_path in URLS_AND_PATHS.items():
    thread = threading.Thread(target=download_page, args=(url, save_path))
    threads.append(thread)
    thread.start()
    # Limit the number of concurrent threads to avoid overloading
    if len(threads) >= 5:
        for thread in threads:
            thread.join()
        threads = []

# Wait for all threads to complete
for thread in threads:
    thread.join()

logging.info("All pages have been saved successfully.")


"""
This is script that uses manual set language. If you wan to use this version, comment out above, uncomment this version and uncomment and set your language code in 
# chrome_options.add_argument("--lang=YOUR_LANGAGE_CODE_HERE") # Add languages: English and MINE

import sys
import os
import logging
import threading
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException, WebDriverException
import time

# Get the current script directory
current_dir = os.path.dirname(os.path.abspath(__file__))
tmp_dir = os.path.join(current_dir, "Tmp")

# Ensure the /Tmp directory exists
if not os.path.exists(tmp_dir):
    os.makedirs(tmp_dir)

# Configure logging
log_path = os.path.join(current_dir, "python_script.log")
logging.basicConfig(
    filename=log_path,
    filemode='w',  # This ensures the log file is overwritten at the start of each run
    level=logging.DEBUG,  # Set logging to DEBUG level to capture more details
    format='%(asctime)s - %(levelname)s - %(message)s'
)

logging.debug("Starting the Python script.")

# Check if the IMDb URL is passed as a parameter
if len(sys.argv) < 2:
    logging.error("IMDb URL was not provided as a parameter.")
    sys.exit("IMDb URL was not provided as a parameter.")

imdb_url = sys.argv[1]
base_url = imdb_url.rstrip('/')

# Generate URLs and their corresponding save paths dynamically, excluding -Main page
URLS_AND_PATHS = {
    f"{base_url}/awards/": os.path.join(tmp_dir, "downpage-UTF8_NO_BOM-Awards.htm"),
    f"{base_url}/bio/": os.path.join(tmp_dir, "downpage-UTF8_NO_BOM-Bio.htm"),
    f"https://www.imdb.com/search/title/?explore=genres&role={base_url.split('/')[-1]}": os.path.join(tmp_dir, "downpage-UTF8_NO_BOM-Genres.htm"),
    f"{base_url}/?showAllCredits=true": os.path.join(tmp_dir, "downpage-UTF8_NO_BOM-Credit.htm")
}

logging.debug(f"Generated URLs and Paths: {URLS_AND_PATHS}")

# Path to chromedriver.exe
chrome_path = os.path.join(current_dir, "chromedriver.exe")

# Ensure the ChromeDriver exists
if not os.path.exists(chrome_path):
    logging.error(f"ChromeDriver not found at path: {chrome_path}")
    sys.exit(f"ChromeDriver not found at path: {chrome_path}")

# Set Chrome options
chrome_options = webdriver.ChromeOptions()
chrome_options.add_argument("--no-sandbox")
chrome_options.add_argument("--disable-dev-shm-usage")
chrome_options.add_argument("--disable-gpu")  # Disable GPU for headless mode stability
chrome_options.add_argument("--headless")  # Running Chrome in headless mode
chrome_options.add_argument("--user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36")
# chrome_options.add_argument("--lang=YOUR_LANGUAGE_CODE_HERE") # Add languages: English and MINE (you can add more, like en,fr etc...)

# Function to download a page
def download_page(imdb_url, output_path, retries=3):
    logging.debug(f"Starting download for URL: {imdb_url}")
    logging.debug(f"Output path: {output_path}")

    attempt = 0
    while attempt < retries:
        attempt += 1
        logging.debug(f"Attempt {attempt} for URL: {imdb_url}")

        try:
            # Start chromedriver.exe
            service = Service(chrome_path)
            driver = webdriver.Chrome(service=service, options=chrome_options)
            logging.info(f"ChromeDriver started from: {chrome_path}")

            try:
                # Open the base URL to set the cookie
                driver.get("https://www.imdb.com")
                cookie = {'name': 'example_cookie', 'value': 'example_value', 'domain': 'imdb.com'}
                driver.add_cookie(cookie)
                logging.debug("Cookie added.")

                # Navigate to the IMDb page
                driver.get(imdb_url)
                logging.info(f"Page {imdb_url} loaded successfully.")

                # Handle "Select Your Preferences" popup with a timeout
                try:
                    popup = WebDriverWait(driver, 3).until(
                        EC.presence_of_element_located((By.XPATH, "//div[contains(@class, 'sc-kDvujY')]"))
                    )
                    logging.info("'Select Your Preferences' popup detected.")
                    accept_button = WebDriverWait(driver, 3).until(
                        EC.element_to_be_clickable((By.XPATH, "//button[@data-testid='accept-button']"))
                    )
                    accept_button.click()
                    logging.info("Clicked on the 'Accept' button to close the popup.")
                except TimeoutException:
                    logging.info("No 'Select Your Preferences' popup appeared or an error occurred. Moving on.")

                # Wait for the page to load
                WebDriverWait(driver, 30).until(
                    EC.presence_of_element_located((By.TAG_NAME, "body"))
                )

                # Click all "More" buttons
                def click_see_more_buttons():
                    while True:
                        buttons = driver.find_elements(By.XPATH, "//span[@class='ipc-btn__text']//span[contains(@class, 'ipc-see-more__text')]")
                        if not buttons:
                            logging.debug("No more 'See More' buttons found.")
                            break
                        for button in buttons:
                            try:
                                driver.execute_script("arguments[0].scrollIntoView(true);", button)  # Scroll to the button to ensure its visible
                                time.sleep(1)
                                button.click()
                                logging.info("Clicked a 'See More' button.")
                                time.sleep(1)  # Short delay to ensure the click is registered
                            except Exception as e:
                                logging.warning(f"Error clicking a 'See More' button: {e}")

                # Click the "See More" buttons
                click_see_more_buttons()

                # Retrieve the full source HTML of the page after all "More" buttons are clicked
                html_source = driver.page_source
                logging.debug(f"HTML source length: {len(html_source)}")

                # Save the HTML to a file
                with open(output_path, 'w', encoding='utf-8') as file:
                    file.write(html_source)
                logging.info(f"HTML saved to file: {output_path}")

                # If successful, break out of the retry loop
                break

            except WebDriverException as e:
                logging.error(f"An error occurred while processing {imdb_url}: {e}")
                driver.quit()

        except Exception as e:
            logging.error(f"An error occurred: {e}")

        finally:
            driver.quit()
            logging.info("Browser closed.")

# Create and start threads for each URL
threads = []
for url, save_path in URLS_AND_PATHS.items():
    thread = threading.Thread(target=download_page, args=(url, save_path))
    threads.append(thread)
    thread.start()
    # Limit the number of concurrent threads to avoid overloading
    if len(threads) >= 5:
        for thread in threads:
            thread.join()
        threads = []

# Wait for all threads to complete
for thread in threads:
    thread.join()

logging.info("All pages have been saved successfully.")

"""
